package webcore.webserver;

import java.io.*;
import java.net.*;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.SecureRandom;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.KeyManager;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.SSLContext;

/****
 * WebServer Socket Factory for SSL web servers
 */
public class WebServerSSLSocketFactory implements WebServerSocketFactoryInterface
{

    final public static boolean NEED_CLIENT_AUTHENTICATION = false;
    final public static String KEY_STORE_TYPE = "JKS";
    final public static String KEY_MANAGER_FACTORY_TYPE = "SunX509";
    final public static String TRUST_MANAGER_FACTORY_TYPE = "SunX509";
    final public static String SSL_VERSION = "TLS";
    final public static String[] CIPHER_SUITES_EXPORT = new String[]
    {
            "SSL_RSA_WITH_RC4_128_MD5",
            "SSL_RSA_WITH_RC4_128_SHA",
            "SSL_RSA_WITH_DES_CBC_SHA",
            "SSL_RSA_WITH_3DES_EDE_CBC_SHA",
            "SSL_RSA_EXPORT_WITH_RC4_40_MD5"
    };

    private SSLContext sslContext = null;


    /****
     * constructs a new SSL Socket Factory
     * @param ksPath path to a java keystore containing the server's certificate
     * and private key; the keystore password and key password must be the same
     * @param pass the keystore password and key password
     * @throws Exception - this method throws a bunch of exceptions most of which
     * indicate a configuration error
     */
    public WebServerSSLSocketFactory(String ksPath, String pass)
        throws GeneralSecurityException, IOException
    {
        setSSLContext(ksPath, pass);
    }

    /**
     * Setting the SSLContext will make this an HTTPS server.  Set the port
     * to 443 if you want to use the default HTTPS port.
     *
     * @param ksPath path to a java keystore containing the server's certificate
     * and private key; the keystore password and key password must be the same
     * @param pass the keystore password and key password
     * @throws Exception - this method throws a bunch of exceptions most of which
     * indicate a configuration error
     */
    public void setSSLContext( String ksPath, String pass )
        throws GeneralSecurityException, IOException
    {
        KeyStore ks = KeyStore.getInstance(KEY_STORE_TYPE);
        FileInputStream fis = new FileInputStream(ksPath);
        try
        {
            ks.load( fis, pass.toCharArray() );
        }
        finally
        {
            fis.close();
        }

        KeyManagerFactory kmf =
            KeyManagerFactory.getInstance(KEY_MANAGER_FACTORY_TYPE);
        kmf.init( ks, pass.toCharArray() );
        KeyManager[] keyManagers = kmf.getKeyManagers();

        TrustManagerFactory tmf =
            TrustManagerFactory.getInstance(TRUST_MANAGER_FACTORY_TYPE);
        tmf.init( ks );
        TrustManager[] trustManagers = tmf.getTrustManagers();

        SecureRandom random = new SecureRandom();

        sslContext = SSLContext.getInstance( SSL_VERSION );
        sslContext.init( keyManagers, trustManagers, random );
    }

    /**
     * Get the SSLContext. This will return null if the context has not
     * been set.
     */
    public SSLContext getSSLContext()
    {
        return sslContext;
    }


    /***
     * Returns an SSL socket listening on the requested port.
     */
    public ServerSocket getServerSocket(int requestedPort)
        throws java.io.IOException
    {
        javax.net.ssl.SSLServerSocket ss = (javax.net.ssl.SSLServerSocket)
            sslContext.getServerSocketFactory().createServerSocket(
                requestedPort);
        ss.setEnabledCipherSuites(CIPHER_SUITES_EXPORT);
        ss.setNeedClientAuth(NEED_CLIENT_AUTHENTICATION);
        return ss;
    }

    public String toString()
    {
        return "SSL Socket Factory";
    }
}
